'use client';

import { useEffect, useState, useCallback } from 'react';
import { DashboardLayout } from '@/components/layouts/dashboard-layout';
import  BalanceCard from '@/components/billing/balance-card';
import { UsageSummary } from '@/components/billing/usage-summary';
import { UsageLogsTable } from '@/components/billing/usage-logs-table';
import { PaymentsSection } from '@/components/billing/payments-section';
import { DateRangeSelector } from '@/components/billing/date-range-selector';
import { Box } from '@radix-ui/themes';
import * as Toast from '@radix-ui/react-toast';
import { getDateRange } from '@/lib/utils';
import { useToast } from '@/components/ui/use-toast';
import { fetchApiEndpoint } from '@/utils/api';
import { useNavigate } from 'react-router-dom';
import { debounce } from '@/utils/debounce';
import { LoadingSpinner } from '@/components/loading-spinner.tsx';
import { BalanceResponse, UsageLogResponse, UsageLogResponseItem, UsageSummaryResponse } from './type';

interface UsageLog {
  id: string;
  timestamp: string;
  characters: number;
  cost: number;
  model: string;
  fileName: string;
  type: string;
}


export default function BillingPage() {
  const { toast } = useToast();
  const navigate = useNavigate();

  const [balance, setBalance] = useState(0);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  const [dateRange, setDateRange] = useState('month');

  const [usageData, setUsageData] = useState({
    totalCharacters: 0,
    totalCost: 0,
    period: 'This Month'
  });

  // Usage Logs State
  const [usageLogs, setUsageLogs] = useState<UsageLog[]>([]);
  const [usageLogsPage, setUsageLogsPage] = useState(1);
  const [usageLogsTotalPages, setUsageLogsTotalPages] = useState(1);

  // Payments State
  const [payments] = useState([]);
  const [paymentsPage, setPaymentsPage] = useState(1);
  const [paymentsTotalPages] = useState(1);
  const [paymentSearchQuery, setPaymentSearchQuery] = useState('');

  const fetchBalance = useCallback(async () => {
    const url = '/billing/balance'
    const options = { method: 'GET' }

    const response = await fetchApiEndpoint<BalanceResponse>(toast, navigate, url, options)

    if(response) {  
      setBalance(parseFloat(response.balance));
    }
  }, [toast, navigate]);

  const fetchUsageSummary = useCallback(async () => {
    const { start, end } = getDateRange(dateRange);

    const url = `/billing/usage-summary/${Math.floor(start.getTime() / 1000)}/${Math.floor(end.getTime() / 1000)}`
    const options = { method: 'GET' }
    const result = await fetchApiEndpoint<Array<UsageSummaryResponse>>(toast, navigate, url, options)


    const periods = {
      today: 'Today',
      week: 'This Week',
      month: 'This Month',
      year: 'This Year',
      all: 'All Time'
    } as const


    if(result) {
      const totalCharacters = result.reduce((sum, item) => sum + parseInt(item.total_characters), 0);
      const totalCost = result.reduce((sum, item) => sum + parseFloat(item.total_cost), 0);
      
      setUsageData({
        totalCharacters: totalCharacters,
        totalCost: totalCost,
        period: periods[dateRange as keyof typeof periods]
      });
    }
  }, [dateRange, toast, navigate]);

  const fetchUsageLogs = useCallback(async () => {
    const { start, end } = getDateRange(dateRange);
    const url = `/billing/usage-logs/${usageLogsPage}/20/${Math.floor(start.getTime() / 1000)}/${Math.floor(end.getTime() / 1000)}`
    const result = await fetchApiEndpoint<UsageLogResponse>(toast, navigate, url, { method: 'GET' })

    if(result) {
      const transformedLogs = result.items.map((log: UsageLogResponseItem) => ({
        id: log.usage_id,
        timestamp: log.created_at.toLocaleString(),
        characters: parseInt(log.total_characters),
        cost: parseFloat(log.total_cost),
        model: log.model,
        fileName: log.original_file_name,
        type: `[${log.model}] ${log.original_file_name}`
      }));

      setUsageLogs(transformedLogs);
      
      // Calculate total pages (assuming 20 items per page)
      const totalItems = result.total || 0;
      setUsageLogsTotalPages(Math.ceil(totalItems / 20));
      setLoading(false);
    }
  }, [dateRange, usageLogsPage, toast, navigate]);

  useEffect(() => {
    fetchBalance();
    fetchUsageSummary();
    fetchUsageLogs();
  }, [fetchBalance, fetchUsageSummary, fetchUsageLogs]);

  const handlePaymentSearch = debounce((query: string) => {
    setPaymentSearchQuery(query);
    setPaymentsPage(1);
    // Implement payment search
  }, 1000);

  if (loading) {
    return (
      <DashboardLayout>
        <LoadingSpinner />
      </DashboardLayout>
    );
  }

  return (
    <DashboardLayout>
      <div className="space-y-8">
        <Box className="space-y-6">
          <BalanceCard balance={balance} />
          <div className="space-y-4">
            <DateRangeSelector value={dateRange} onChange={setDateRange} />
            <UsageSummary {...usageData} />
          </div>
        </Box>

        <div className="space-y-4">
          <h3 className="text-2xl font-medium text-gray-900 dark:text-gray-100">Usage Logs</h3>
          <UsageLogsTable
            logs={usageLogs}
            currentPage={usageLogsPage}
            totalPages={usageLogsTotalPages}
            onPageChange={setUsageLogsPage}
          />
        </div>

        <PaymentsSection
          payments={payments}
          currentPage={paymentsPage}
          totalPages={paymentsTotalPages}
          searchQuery={paymentSearchQuery}
          onSearchChange={handlePaymentSearch}
          onPageChange={setPaymentsPage}
        />

        {error && (
          <Toast.Provider>
            <Toast.Root className="bg-white dark:bg-gray-800 rounded-md shadow-lg p-4 border border-gray-200 dark:border-gray-700" open={!!error} onOpenChange={() => setError(null)}>
              <Toast.Title className="font-medium text-red-600 dark:text-red-400">Error</Toast.Title>
              <Toast.Description className="text-gray-600 dark:text-gray-400 mt-1">{error}</Toast.Description>
            </Toast.Root>
            <Toast.Viewport className="fixed bottom-0 right-0 p-6 m-0 flex flex-col gap-2 w-[390px] max-w-[100vw] list-none z-50" />
          </Toast.Provider>
        )}
      </div>
    </DashboardLayout>
  );
}
